@@ -42,12 +42,12 @@
 #include <X11/Xatom.h>
 
 #include "mate-panel-applet.h"
+#include "panel-applet-private.h"
 #include "mate-panel-applet-factory.h"
 #include "mate-panel-applet-marshal.h"
 #include "mate-panel-applet-enums.h"
-#if GTK_CHECK_VERSION (3, 18, 0)
 #include "panel-plug-private.h"
-#endif
+
 #define MATE_PANEL_APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET, MatePanelAppletPrivate))
 
 struct _MatePanelAppletPrivate {
@@ -55,6 +55,8 @@ struct _MatePanelAppletPrivate {
 	GtkWidget         *applet;
 	GDBusConnection   *connection;
 
+	gboolean           out_of_process;
+
 	char              *id;
 	GClosure          *closure;
 	char              *object_path;
@@ -69,9 +71,6 @@ struct _MatePanelAppletPrivate {
 	MatePanelAppletOrient  orient;
 	guint              size;
 	char              *background;
-#if !GTK_CHECK_VERSION (3, 18, 0)
-	GtkWidget         *background_widget;
-#endif
 
 	int                previous_width;
 	int                previous_height;
@@ -97,6 +96,7 @@ static guint mate_panel_applet_signals[LAST_SIGNAL] = { 0 };
 
 enum {
 	PROP_0,
+	PROP_OUT_OF_PROCESS,
 	PROP_ID,
 	PROP_CLOSURE,
 	PROP_CONNECTION,
@@ -136,7 +136,7 @@ static const gchar panel_menu_ui[] =
 	"</ui>\n";
 
 static const GtkActionEntry menu_entries[] = {
-	{ "Remove", GTK_STOCK_REMOVE, N_("_Remove From Panel"),
+	{ "Remove", "list-remove", N_("_Remove From Panel"),
 	  NULL, NULL,
 	  G_CALLBACK (mate_panel_applet_menu_cmd_remove) },
 	{ "Move", NULL, N_("_Move"),
@@ -269,6 +269,13 @@ mate_panel_applet_size_hints_changed (MatePanelApplet *applet,
 	return FALSE;
 }
 
+/**
+ * mate_panel_applet_set_size_hints: 
+ * @applet: applet
+ * @size_hints: (array length=n_elements): List of integers
+ * @n_elements: Length of @size_hints
+ * @base_size: base_size
+ */
 void
 mate_panel_applet_set_size_hints (MatePanelApplet *applet,
 			     const int   *size_hints,
@@ -803,13 +810,10 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
 	gdk_window_get_origin (gtk_widget_get_window (widget),
 			       &menu_x, &menu_y);
-#if GTK_CHECK_VERSION (3, 20, 0)
+
 	device = gdk_seat_get_pointer (gdk_display_get_default_seat (gtk_widget_get_display (widget)));
 	gdk_window_get_device_position (gtk_widget_get_window (widget), device, &pointer_x, &pointer_y, NULL);
-#else
-	device = gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gtk_widget_get_display (widget)));
-	gdk_window_get_device_position (gtk_widget_get_window (widget), device, &pointer_x, &pointer_y, NULL);
-#endif
+
 	gtk_widget_get_allocation (widget, &allocation);
 
 	menu_x += allocation.x;
@@ -830,9 +834,9 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 					       allocation.width - requisition.width);
 			}
 		}
-		menu_x = MIN (menu_x, gdk_screen_get_width (screen) - requisition.width);
+		menu_x = MIN (menu_x, WidthOfScreen (gdk_x11_screen_get_xscreen (screen)) - requisition.width);
 
-		if (menu_y > gdk_screen_get_height (screen) / 2)
+		if (menu_y > HeightOfScreen (gdk_x11_screen_get_xscreen (screen)) / 2)
 			menu_y -= requisition.height;
 		else
 			menu_y += allocation.height;
@@ -840,9 +844,9 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 		if (pointer_y < allocation.height &&
 		    requisition.height < pointer_y)
 			menu_y += MIN (pointer_y, allocation.height - requisition.height);
-		menu_y = MIN (menu_y, gdk_screen_get_height (screen) - requisition.height);
+		menu_y = MIN (menu_y, HeightOfScreen (gdk_x11_screen_get_xscreen (screen)) - requisition.height);
 
-		if (menu_x > gdk_screen_get_width (screen) / 2)
+		if (menu_x > WidthOfScreen (gdk_x11_screen_get_xscreen (screen)) / 2)
 			menu_x -= requisition.width;
 		else
 			menu_x += allocation.width;
@@ -900,13 +904,19 @@ mate_panel_applet_can_focus (GtkWidget *widget)
 
 /* Taken from libmatecomponentui/matecomponent/matecomponent-plug.c */
 static gboolean
-mate_panel_applet_button_event (GtkWidget      *widget,
+mate_panel_applet_button_event (MatePanelApplet      *applet,
 			   GdkEventButton *event)
 {
+	GtkWidget *widget;
 	GdkWindow *window;
 	GdkWindow *socket_window;
 	XEvent     xevent;
 
+	if (!applet->priv->out_of_process)
+		return FALSE;
+
+	widget = applet->priv->plug;
+
 	if (!gtk_widget_is_toplevel (widget))
 		return FALSE;
 
@@ -914,7 +924,6 @@ mate_panel_applet_button_event (GtkWidget      *widget,
 	socket_window = gtk_plug_get_socket_window (GTK_PLUG (widget));
 
 	if (event->type == GDK_BUTTON_PRESS) {
-#if GTK_CHECK_VERSION (3, 20, 0)
 		GdkDisplay *display;
 		GdkSeat *seat;
 
@@ -929,16 +938,6 @@ mate_panel_applet_button_event (GtkWidget      *widget,
 		 * We don't want to hog the pointer on our parent.
 		 */
 		gdk_seat_ungrab (seat);
-#else
-		xevent.xbutton.type = ButtonPress;
-
-		/* X does an automatic pointer grab on button press
-		 * if we have both button press and release events
-		 * selected.
-		 * We don't want to hog the pointer on our parent.
-		 */
-		gdk_device_ungrab (event->device, GDK_CURRENT_TIME);
-#endif
 	} else {
 		xevent.xbutton.type = ButtonRelease;
 	}
@@ -990,7 +989,7 @@ mate_panel_applet_button_press (GtkWidget      *widget,
 		return TRUE;
 	}
 
-	return mate_panel_applet_button_event (applet->priv->plug, event);
+	return mate_panel_applet_button_event (applet, event);
 }
 
 static gboolean
@@ -999,7 +998,7 @@ mate_panel_applet_button_release (GtkWidget      *widget,
 {
 	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
 
-	return mate_panel_applet_button_event (applet->priv->plug, event);
+	return mate_panel_applet_button_event (applet, event);
 }
 
 static gboolean
@@ -1010,29 +1009,26 @@ mate_panel_applet_popup_menu (GtkWidget *widget)
 	return TRUE;
 }
 
-#if !GTK_CHECK_VERSION (3, 18, 0)
 static void
 mate_panel_applet_get_preferred_width (GtkWidget *widget,
 				       int       *minimum_width,
 				       int       *natural_width)
 {
-	int focus_width = 0;
+	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
+	gint scale;
 
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->get_preferred_width (widget,
 										minimum_width,
 										natural_width);
-	if (!mate_panel_applet_can_focus (widget))
-		return;
 
-	/* We are deliberately ignoring focus-padding here to
-	 * save valuable panel real estate.
-	 */
-	gtk_widget_style_get (widget,
-			      "focus-line-width", &focus_width,
-			      NULL);
-
-	*minimum_width += 2 * focus_width;
-	*natural_width += 2 * focus_width;
+	if (applet->priv->out_of_process) {
+		/* Out-of-process applets end up scaled up doubly. We divide by the scale factor to ensure
+		 * they are back at their own intended size.
+		 */
+		scale = gtk_widget_get_scale_factor (widget);
+		*minimum_width /= scale;
+		*natural_width /= scale;
+	}
 }
 
 static void
@@ -1040,25 +1036,22 @@ mate_panel_applet_get_preferred_height (GtkWidget *widget,
 					int       *minimum_height,
 					int       *natural_height)
 {
-	int focus_width = 0;
+	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
+	gint scale;
 
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->get_preferred_height (widget,
-										 minimum_height,
-										 natural_height);
-	if (!mate_panel_applet_can_focus (widget))
-		return;
+										minimum_height,
+										natural_height);
 
-	/* We are deliberately ignoring focus-padding here to
-	 * save valuable panel real estate.
-	 */
-	gtk_widget_style_get (widget,
-			      "focus-line-width", &focus_width,
-			      NULL);
-
-	*minimum_height += 2 * focus_width;
-	*natural_height += 2 * focus_width;
+	if (applet->priv->out_of_process) {
+		/* Out-of-process applets end up scaled up doubly. We divide by the scale factor to ensure
+		 * they are back at their own intended size.
+		 */
+		scale = gtk_widget_get_scale_factor (widget);
+		*minimum_height /= scale;
+		*natural_height /= scale;
+	}
 }
-#endif
 
 static GtkSizeRequestMode
 mate_panel_applet_get_request_mode (GtkWidget *widget)
@@ -1066,6 +1059,9 @@ mate_panel_applet_get_request_mode (GtkWidget *widget)
 	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
 	MatePanelAppletOrient orientation;
 
+	if (applet->priv->out_of_process)
+		return GTK_SIZE_REQUEST_CONSTANT_SIZE;
+
 	orientation = mate_panel_applet_get_orient (applet);
 	if (orientation == MATE_PANEL_APPLET_ORIENT_UP ||
 	    orientation == MATE_PANEL_APPLET_ORIENT_DOWN)
@@ -1082,36 +1078,19 @@ mate_panel_applet_size_allocate (GtkWidget     *widget,
 	GtkBin        *bin;
 	GtkWidget     *child;
 	int            border_width;
-#if !GTK_CHECK_VERSION (3, 18, 0)
-	int            focus_width = 0;
-#endif
 	MatePanelApplet   *applet;
 
 	if (!mate_panel_applet_can_focus (widget)) {
 		GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->size_allocate (widget, allocation);
 	} else {
-#if !GTK_CHECK_VERSION (3, 18, 0)
-		/*
-		 * We are deliberately ignoring focus-padding here to
-		 * save valuable panel real estate.
-		 */
-		gtk_widget_style_get (widget,
-				      "focus-line-width", &focus_width,
-				      NULL);
-#endif
 
 		border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
 		gtk_widget_set_allocation (widget, allocation);
 		bin = GTK_BIN (widget);
 
-#if GTK_CHECK_VERSION (3, 18, 0)
 		child_allocation.x = 0;
 		child_allocation.y = 0;
-#else
-		child_allocation.x = focus_width;
-		child_allocation.y = focus_width;
-#endif
 
 		child_allocation.width  = MAX (allocation->width  - border_width * 2, 0);
 		child_allocation.height = MAX (allocation->height - border_width * 2, 0);
@@ -1123,11 +1102,6 @@ mate_panel_applet_size_allocate (GtkWidget     *widget,
 						child_allocation.width,
 						child_allocation.height);
 
-#if !GTK_CHECK_VERSION (3, 18, 0)
-		child_allocation.width  = MAX (child_allocation.width  - 2 * focus_width, 0);
-		child_allocation.height = MAX (child_allocation.height - 2 * focus_width, 0);
-#endif
-
 		child = gtk_bin_get_child (bin);
 		if (child)
 			gtk_widget_size_allocate (child, &child_allocation);
@@ -1148,9 +1122,6 @@ static gboolean mate_panel_applet_draw(GtkWidget* widget, cairo_t* cr)
 {
 	GtkStyleContext *context;
 	int border_width;
-#if !GTK_CHECK_VERSION (3, 18, 0)
-	int focus_width = 0;
-#endif
 	gdouble x, y, width, height;
 
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->draw(widget, cr);
@@ -1161,16 +1132,6 @@ static gboolean mate_panel_applet_draw(GtkWidget* widget, cairo_t* cr)
 	width = gtk_widget_get_allocated_width (widget);
 	height = gtk_widget_get_allocated_height (widget);
 
-#if !GTK_CHECK_VERSION (3, 18, 0)
-	/*
-	 * We are deliberately ignoring focus-padding here to
-	 * save valuable panel real estate.
-	 */
-	gtk_widget_style_get (widget,
-		"focus-line-width", &focus_width,
-		NULL);
-#endif
-
 	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
 	x = 0;
@@ -1350,8 +1311,10 @@ mate_panel_applet_get_pattern_from_pixmap (MatePanelApplet *applet,
 
 	width = gdk_window_get_width(window);
 	height = gdk_window_get_height(window);
-	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
-
+	surface = gdk_window_create_similar_surface (window,
+	                            CAIRO_CONTENT_COLOR_ALPHA,
+	                            width,
+	                            height);
 	gdk_error_trap_push ();
 	cr = cairo_create (surface);
 	cairo_set_source_surface (cr, background, -x, -y);
@@ -1526,59 +1489,60 @@ mate_panel_applet_move_focus_out_of_applet (MatePanelApplet      *applet,
 	applet->priv->moving_focus_out = FALSE;
 }
 
-#if GTK_CHECK_VERSION (3, 18, 0)
 static void
 mate_panel_applet_change_background(MatePanelApplet *applet,
 				    MatePanelAppletBackgroundType type,
 				    GdkRGBA* color,
 				    cairo_pattern_t *pattern)
 {
-	GdkWindow* window = gtk_widget_get_window (applet->priv->plug);
+	GtkStyleContext* context;
+	GdkWindow* window;
+
+	if (applet->priv->out_of_process)
+		window = gtk_widget_get_window (GTK_WIDGET (applet->priv->plug));
+	else
+		window = gtk_widget_get_window (GTK_WIDGET (applet));
+
 	gtk_widget_set_app_paintable(GTK_WIDGET(applet),TRUE);
-	_mate_panel_applet_apply_css(GTK_WIDGET(applet->priv->plug),type);
+
+	if (applet->priv->out_of_process)
+		_mate_panel_applet_apply_css(GTK_WIDGET(applet->priv->plug),type);
+
 	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		gdk_window_set_background_pattern(window,NULL);
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gdk_window_set_background_rgba(window,color);
-		gtk_widget_queue_draw (applet->priv->plug); /*change the bg right away always */
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		gdk_window_set_background_pattern(window,pattern);
-		gtk_widget_queue_draw (applet->priv->plug); /*change the bg right away always */
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
+		case PANEL_NO_BACKGROUND:
+			if (applet->priv->out_of_process){
+				pattern = cairo_pattern_create_rgba (0,0,0,0);     /* Using NULL here breaks transparent */
+				gdk_window_set_background_pattern(window,pattern); /* backgrounds set by GTK theme */
+			}
+			break;
+		case PANEL_COLOR_BACKGROUND:
+			if (applet->priv->out_of_process){
+				gdk_window_set_background_rgba(window,color);
+				gtk_widget_queue_draw (applet->priv->plug); /*change the bg right away always */
+			}
+			break;
+		case PANEL_PIXMAP_BACKGROUND:
+			if (applet->priv->out_of_process){
+				gdk_window_set_background_pattern(window,pattern);
+				gtk_widget_queue_draw (applet->priv->plug); /*change the bg right away always */
+			}
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+	}
+
+	if (applet->priv->out_of_process){
+		context = gtk_widget_get_style_context (GTK_WIDGET(applet->priv->plug));
+		if (applet->priv->orient == MATE_PANEL_APPLET_ORIENT_UP ||
+			applet->priv->orient == MATE_PANEL_APPLET_ORIENT_DOWN){
+			gtk_style_context_add_class(context,"horizontal");
+		}
+		else {
+			gtk_style_context_add_class(context,"vertical");
+		}
 	}
 }
-#else
-static void
-mate_panel_applet_change_background(MatePanelApplet *applet,
-				    MatePanelAppletBackgroundType type,
-				    GdkRGBA* color,
-				    cairo_pattern_t *pattern)
-{
-	GdkWindow* window = gtk_widget_get_window(GTK_WIDGET(applet));
-	gtk_widget_set_app_paintable(GTK_WIDGET(applet),TRUE);
-	_mate_panel_applet_apply_css(GTK_WIDGET(applet->priv->plug),type);
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		gdk_window_set_background_pattern(window,NULL);
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gdk_window_set_background_rgba(window,color);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		gdk_window_set_background_pattern(window,pattern);
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-}
-#endif
 
 static void
 mate_panel_applet_get_property (GObject    *object,
@@ -1589,6 +1553,9 @@ mate_panel_applet_get_property (GObject    *object,
 	MatePanelApplet *applet = MATE_PANEL_APPLET (object);
 
 	switch (prop_id) {
+	case PROP_OUT_OF_PROCESS:
+		g_value_set_boolean (value, applet->priv->out_of_process);
+		break;
 	case PROP_ID:
 		g_value_set_string (value, applet->priv->id);
 		break;
@@ -1647,6 +1614,9 @@ mate_panel_applet_set_property (GObject      *object,
 	MatePanelApplet *applet = MATE_PANEL_APPLET (object);
 
 	switch (prop_id) {
+	case PROP_OUT_OF_PROCESS:
+		applet->priv->out_of_process = g_value_get_boolean (value);
+		break;
 	case PROP_ID:
 		applet->priv->id = g_value_dup_string (value);
 		break;
@@ -1776,17 +1746,21 @@ static void _mate_panel_applet_prepare_css (GtkStyleContext *context)
 
 	provider = gtk_css_provider_new ();
 	gtk_css_provider_load_from_data (provider,
-					 ".mate-custom-panel-background{\n"
+					 "#PanelPlug {\n"
+					 " background-repeat: no-repeat;\n" /*disable in gtk theme features */
+					 " background-size: cover; "        /*that don't work on panel-toplevel */
+					 " }\n"
+					 ".mate-custom-panel-background{\n" /*prepare CSS for user set theme */
 					 " background-color: rgba (0, 0, 0, 0);\n"
 					 " background-image: none;\n"
 					 "}",
 					 -1, NULL);
+
 	gtk_style_context_add_provider (context,
 					GTK_STYLE_PROVIDER (provider),
 					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 	g_object_unref (provider);
 }
-
 static void
 mate_panel_applet_init (MatePanelApplet *applet)
 {
@@ -1813,10 +1787,29 @@ mate_panel_applet_init (MatePanelApplet *applet)
 	gtk_ui_manager_add_ui_from_string (applet->priv->ui_manager,
 					   panel_menu_ui, -1, NULL);
 
+	gtk_widget_set_events (GTK_WIDGET (applet),
+			       GDK_BUTTON_PRESS_MASK |
+			       GDK_BUTTON_RELEASE_MASK);
+}
 
+static GObject *
+mate_panel_applet_constructor (GType                  type,
+                          guint                  n_construct_properties,
+                          GObjectConstructParam *construct_properties)
+{
+	GObject     *object;
+	MatePanelApplet *applet;
 
+	object = G_OBJECT_CLASS (mate_panel_applet_parent_class)->constructor (type,
+	                                                                  n_construct_properties,
+	                                                                  construct_properties);
+	applet = MATE_PANEL_APPLET (object);
+
+	if (!applet->priv->out_of_process)
+		return object;
 
 	applet->priv->plug = gtk_plug_new (0);
+
 	GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(applet->priv->plug));
 	GdkVisual *visual = gdk_screen_get_rgba_visual(screen);
 	gtk_widget_set_visual(GTK_WIDGET(applet->priv->plug), visual);
@@ -1828,16 +1821,14 @@ mate_panel_applet_init (MatePanelApplet *applet)
 	_mate_panel_applet_prepare_css(context);
 
 	g_signal_connect_swapped (G_OBJECT (applet->priv->plug), "embedded",
-				  G_CALLBACK (mate_panel_applet_setup),
-				  applet);
+		                      G_CALLBACK (mate_panel_applet_setup),
+		                      applet);
+ 
+ 	gtk_container_add (GTK_CONTAINER (applet->priv->plug), GTK_WIDGET (applet));
 
-	gtk_widget_set_events (GTK_WIDGET (applet),
-			       GDK_BUTTON_PRESS_MASK |
-			       GDK_BUTTON_RELEASE_MASK);
-
-	gtk_container_add (GTK_CONTAINER (applet->priv->plug), GTK_WIDGET (applet));
+	return object;
 }
-
+ 
 static void
 mate_panel_applet_constructed (GObject* object)
 {
@@ -1860,16 +1851,15 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 
 	gobject_class->get_property = mate_panel_applet_get_property;
 	gobject_class->set_property = mate_panel_applet_set_property;
+	gobject_class->constructor = mate_panel_applet_constructor;
 	gobject_class->constructed = mate_panel_applet_constructed;
 	klass->move_focus_out_of_applet = mate_panel_applet_move_focus_out_of_applet;
 	klass->change_background = mate_panel_applet_change_background;
 	widget_class->button_press_event = mate_panel_applet_button_press;
 	widget_class->button_release_event = mate_panel_applet_button_release;
 	widget_class->get_request_mode = mate_panel_applet_get_request_mode;
-#if !GTK_CHECK_VERSION (3, 18, 0)
 	widget_class->get_preferred_width = mate_panel_applet_get_preferred_width;
 	widget_class->get_preferred_height = mate_panel_applet_get_preferred_height;
-#endif
 	widget_class->draw = mate_panel_applet_draw;
 	widget_class->size_allocate = mate_panel_applet_size_allocate;
 	widget_class->focus = mate_panel_applet_focus;
@@ -1880,6 +1870,14 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 
 	g_type_class_add_private (klass, sizeof (MatePanelAppletPrivate));
 
+	g_object_class_install_property (gobject_class,
+	                  PROP_OUT_OF_PROCESS,
+	                  g_param_spec_boolean ("out-of-process",
+	                               "out-of-process",
+	                               "out-of-process",
+	                                TRUE,
+	                                G_PARAM_CONSTRUCT_ONLY |
+	                                G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_class,
 					 PROP_ID,
 					 g_param_spec_string ("id",
@@ -2020,9 +2018,7 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK, GTK_DIR_TAB_FORWARD);
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
 
-#if GTK_CHECK_VERSION (3, 20, 0)
 	gtk_widget_class_set_css_name (widget_class, "PanelApplet");
-#endif
 }
 
 GtkWidget* mate_panel_applet_new(void)
@@ -2248,17 +2244,13 @@ _mate_panel_applet_setup_x_error_handler (void)
 	_x_error_func = XSetErrorHandler (_x_error_handler);
 }
 
-/**
- * mate_panel_applet_factory_main:
- * @factory_id: Factory ID.
- * @out_process: If the factory is on a separate process or not.
- * @applet_type: GType of the applet this factory creates.
- * @callback: (scope call): Callback to be called when a new applet is to be created.
- * @data: (closure): Callback data.
- *
- * Returns: 0 on success, 1 if there is an error.
- */
-int mate_panel_applet_factory_main(const gchar* factory_id, gboolean out_process, GType applet_type, MatePanelAppletFactoryCallback callback, gpointer user_data)
+static int
+_mate_panel_applet_factory_main_internal (const gchar               *factory_id,
+				     gboolean                   out_process,
+				     GType                      applet_type,
+				     MatePanelAppletFactoryCallback callback,
+					 gpointer                   user_data)
+	
 {
 	MatePanelAppletFactory* factory;
 	GClosure* closure;
@@ -2273,7 +2265,7 @@ int mate_panel_applet_factory_main(const gchar* factory_id, gboolean out_process
 	}
 
 	closure = g_cclosure_new(G_CALLBACK(callback), user_data, NULL);
-	factory = mate_panel_applet_factory_new(factory_id, applet_type, closure);
+	factory = mate_panel_applet_factory_new(factory_id, out_process,  applet_type, closure);
 	g_closure_unref(closure);
 
 	if (mate_panel_applet_factory_register_service(factory))
@@ -2292,6 +2284,47 @@ int mate_panel_applet_factory_main(const gchar* factory_id, gboolean out_process
 	return 1;
 }
 
+/**
+ * mate_panel_applet_factory_main:
+ * @out_process: boolean, dummy to support applets sending it
+ * @factory_id: Factory ID.
+ * @applet_type: GType of the applet this factory creates.
+ * @callback: (scope call): Callback to be called when a new applet is to be created.
+ * @data: (closure): Callback data.
+ *
+ * Returns: 0 on success, 1 if there is an error.
+ */
+int
+mate_panel_applet_factory_main (const gchar               *factory_id,
+               gboolean                   out_process, /*Dummy to support applets w issues with this */
+			   GType                      applet_type,
+			   MatePanelAppletFactoryCallback callback,
+			   gpointer                   user_data)
+{
+	return _mate_panel_applet_factory_main_internal (factory_id, TRUE, applet_type,
+						    callback, user_data);
+}
+
+/**
+ * mate_panel_applet_factory_setup_in_process: (skip)
+ * @factory_id: Factory ID.
+ * @applet_type: GType of the applet this factory creates.
+ * @callback: (scope call): Callback to be called when a new applet is to be created.
+ * @data: (closure): Callback data.
+ *
+ * Returns: 0 on success, 1 if there is an error.
+ */
+int
+mate_panel_applet_factory_setup_in_process (const gchar               *factory_id,
+				       GType                      applet_type,
+				       MatePanelAppletFactoryCallback callback,
+				       gpointer                   user_data)
+{
+	return _mate_panel_applet_factory_main_internal (factory_id, FALSE, applet_type,
+						    callback, user_data);
+}
+
+
 /**
  * mate_panel_applet_set_background_widget:
  * @applet: a #PanelApplet.
@@ -2304,36 +2337,19 @@ int mate_panel_applet_factory_main(const gchar* factory_id, gboolean out_process
  * Deprecated: 3.20: Do not use this API. Since 3.20 this function does nothing.
  **/
 
-#if GTK_CHECK_VERSION (3, 18, 0)
 void
 mate_panel_applet_set_background_widget (MatePanelApplet *applet,
 				    GtkWidget   *widget)
 {
 }
-#else
-void
-mate_panel_applet_set_background_widget (MatePanelApplet *applet,
-				    GtkWidget   *widget)
-{
-	applet->priv->background_widget = widget;
-
-	if (widget && gtk_widget_get_realized (widget)) {
-		MatePanelAppletBackgroundType  type;
-		GdkRGBA                    color;
-		cairo_pattern_t           *pattern;
-		type = mate_panel_applet_get_background (applet, &color, &pattern);
-		_mate_panel_applet_apply_css(widget,type);
-
-		if (type == PANEL_PIXMAP_BACKGROUND)
-			cairo_pattern_destroy (pattern);
-	}
-}
-#endif
 
 guint32
 mate_panel_applet_get_xid (MatePanelApplet *applet,
 		      GdkScreen   *screen)
 {
+	if (applet->priv->out_of_process == FALSE)
+		return 0;
+
 	gtk_window_set_screen (GTK_WINDOW (applet->priv->plug), screen);
 	gtk_widget_show (applet->priv->plug);
 
@@ -2345,3 +2361,19 @@ mate_panel_applet_get_object_path (MatePanelApplet *applet)
 {
 	return applet->priv->object_path;
 }
+
+GtkWidget *
+mate_panel_applet_get_applet_widget (const gchar *factory_id,
+                                guint        uid)
+{
+	GtkWidget *widget;
+
+	widget = mate_panel_applet_factory_get_applet_widget (factory_id, uid);
+	if (!widget) {
+		return NULL;
+	}
+
+	mate_panel_applet_setup (MATE_PANEL_APPLET (widget));
+
+	return widget;
+}
